function x = congrad( x0, f, df, Gs, tol, maxit )
% Anton Evgrafov, 28 Sep 2000
% Conjugate gradient algorithm due to Fketcher-Reeves-Polak-Ribiere

%inputs
% x0    -- initial iteration
% f     -- objective function
% df    -- gradiant of objective function
% Gs    -- constraints to maintan (all( Gs < 0 ))
% tol   -- tolerance of the termination criteria
% maxit -- maximal number of iterations

x = x0;
fx = feval( f, x );
xi = -feval( df, x );
g = xi;
h = g;

lambda = 1;

it_no = 1;

while( it_no <= maxit )
  f_old = fx;
  x_old = x;
  %perform a line-search
  [fx, x, lambda] = linmin( x_old, xi, f, df, Gs, lambda );
  if( 2*norm(  x_old - x ) <= tol*(norm(x_old) + norm(x) + eps) )
  %if( 2*abs( f_old - fx ) <= tol*( abs(f_old) + abs(fx) + eps ) )
    return;
  end %if
  
  xi = -feval( df, x );
  gg = g'*g;
  
  dgg = xi'*xi; % Fletcher-Reeves
  %dgg = (xi-g)'*xi; %Polak-Ribiere -- it doesn't work with
  %Armijo's line - search.... ;-((
  
  if( gg == 0 )
    return;
  end %if
  
  gam = dgg/gg;
  g = xi;
  xi = g + gam * h;
  h = xi;
  
  it_no = it_no + 1;
  
  %sprintf( '%d:  %f %f', it_no, x(1), x(2) )
end %while

warning( 'Iterations limit exceed...' );

function [f_new, x_new, lambda_new] = linmin( x, xi, f, df, Gs, lambda )
%Armijo search parameters
epsilon = 0.2;
alpha = 2;

fx = feval( f, x );
lambda_new = 0;
x_new = x;
f_new = fx;

%Check for the feasibility first
if( ~all( feval( Gs, x ) < 0 ) )
  warning( 'Out of feasible region' );
  return;
end

% feasibility first!!!
while( ~all( feval( Gs, x + lambda * xi ) < 0 ) )
  lambda = lambda / alpha;
  if( abs( lambda ) <= eps )
    x_new = x + lambda * xi;
    f_new = feval( f, x_new );
    lambda_new = lambda;
    return;
  end
end

dfx = epsilon * xi' * feval( df, x );
x_new = x + lambda * xi;
f_new = feval( f, x_new );
if( f_new <= fx + lambda * dfx )
  %we can try to increase step-size
  while( 1 )
    x_new1 = x + alpha * lambda * xi;
    %if x_new1 still feasible?
    if( ~all( feval( Gs, x_new1 ) < 0 ) )
      % increased step-size is inaccessible... ;-(
      lambda_new = lambda;
      return;
    end
    
    f_new1 = feval( f, x_new1 );
    if( f_new1 > fx + lambda * dfx )
      % ok, that's it
      lambda_new = lambda;
      return;
    end
    
    % try to increase even more...
    x_new = x_new1;
    f_new = f_new1;
    lambda = alpha * lambda;
  end
else
  %we need to decrease step-size
  while( 1 )
    lambda = lambda / alpha;
    x_new = x + lambda * xi;

    f_new = feval( f, x_new );
    if( f_new <= fx + lambda * dfx )
      % ok, that's it
      lambda_new = lambda;
      return;
    end    
  end
end

